/*
 *  CsvJdbc - a JDBC driver for CSV files
 *  Copyright (C) 2001  Jonathan Ackerman
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.relique.jdbc.csv;

import java.util.Vector;
import java.util.StringTokenizer;

/**
 *This is a very crude and SQL simple parser used by the Csv JDBC driver. It
 * only handles SELECT statements in the format "SELECT xxx,yyy,zzz FROM fffff"
 *
 * @author     Jonathan Ackerman
 * @created    25 November 2001
 * @version    $Id: SqlParser.java,v 1.2 2001/12/01 22:35:13 jackerm Exp $
 */
public class SqlParser
{
  /**
   *Description of the Field
   *
   * @since
   */
   
 
    public static final String CREATE_TABLE = "create_table";
    public static final String INSERT = "insert";
    public static final String UPDATE = "update";
    public static final String SELECT = "select";


  public String tableName;

  public String whereStatement;
  
  public String sqlType;;
  
  /**
   *Description of the Field
   *
   * @since
   */
  

  public String[] columnNames;
  
  public String[] columnValues;  

  public String[] columnWhereNames;

  public String[] columnWhereValues;


  /**
   *Gets the tableName attribute of the SqlParser object
   *
   * @return    The tableName value
   * @since
   */
  public String getTableName()
  {
    return tableName;
  }


  /**
   *Gets the columnNames attribute of the SqlParser object
   *
   * @return    The columnNames value
   * @since
   */
  public String[] getColumnNames()
  {
    return columnNames;
  }

  public String[] getWhereColumnNames()
  {
    return columnWhereNames;
  }
  
  
  public String[] getWhereColumnValues()
  {
    return columnWhereValues;
  }


  /**
   *Description of the Method
   *
   * @param  sql            Description of Parameter
   * @exception  Exception  Description of Exception
   * @since
   */
  public void parse(String sql) throws Exception
  {
    tableName = null;
    columnNames = new String[0];
	 columnValues = new String[0];
	 columnWhereNames = new String[0];
	 columnWhereValues = new String[0];
	 whereStatement=null;
	 sqlType=null;
	 sql=sql.trim();

    String upperSql = sql.toUpperCase();

    if (!upperSql.startsWith("SELECT "))
    {
    	if (!upperSql.startsWith("INSERT ")) {
    		if (!upperSql.startsWith("UPDATE ")) {
    			if (upperSql.indexOf("CREATE TABLE ")==1)
	      		throw new Exception("Malformed SQL. Wrong SQL statement.");
	      	else {
  		    		sqlType=CREATE_TABLE;
  		    		int createPos = upperSql.indexOf("CREATE TABLE");
  		    		int tableStartPos = upperSql.indexOf("(");
  		    		int tableEndPos = upperSql.lastIndexOf(")");
  		    		tableName = sql.substring(createPos+12, tableStartPos).trim();
					String createString = sql.substring(tableStartPos+1, tableEndPos).trim();
					StringTokenizer tokenizerCreate = new StringTokenizer(createString.toUpperCase(), "\n");
					Vector setColumnNames = new Vector();
	    	
					while (tokenizerCreate.hasMoreTokens())
   	 				{
      				String strToken = tokenizerCreate.nextToken().trim();
      				int delimiter = strToken.indexOf(" ");
      				if(strToken.toUpperCase().indexOf("CONSTRAINT")==-1) {
      					setColumnNames.add(strToken.substring(0,delimiter).trim());
      				}	
    					}
					columnNames = new String[setColumnNames.size()];
    				setColumnNames.copyInto(columnNames);
    				}
    			}		
	      else{
	      	if (upperSql.lastIndexOf(" SET ") == -1)
      				throw new Exception("Malformed SQL. Missing SET statement.");
    		sqlType=UPDATE;
    		int updatePos = upperSql.indexOf("UPDATE");
    		int setPos = upperSql.indexOf(" SET ");
    		int equalPos = upperSql.indexOf("=");
    		int wherePos = upperSql.indexOf(" WHERE ");
    		tableName = sql.substring(updatePos + 6, setPos).trim();
			
    		String setString = sql.substring(setPos+5, wherePos);
//    		int lastBracketPos = upperSql.indexOf(")");
			StringTokenizer tokenizerSet = new StringTokenizer(setString.toUpperCase(), ",");
			Vector setNames = new Vector();
	    	Vector setValues = new Vector();
	    	
			while (tokenizerSet.hasMoreTokens())
    		{
      		String strToken = tokenizerSet.nextToken();
      		int delimiter = strToken.indexOf("=");
      		setNames.add(strToken.substring(0,delimiter).trim());
      		setValues.add(strToken.substring(delimiter+1).trim());
    		}
    		
			
			columnNames = new String[setNames.size()];
	    	columnValues = new String[setValues.size()];
    		setNames.copyInto(columnNames);
    		setValues.copyInto(columnValues);
    		
    	if(wherePos!=-1)
    	 {
    	 	String strWhere=sql.substring(wherePos+6).trim();
    	 	Vector whereCols = new Vector();
    	 	Vector whereValues = new Vector();
    		StringTokenizer tokenizerWhere = new StringTokenizer(strWhere, ",");

    	while (tokenizerWhere.hasMoreTokens())
    	{
      	String strToken = tokenizerWhere.nextToken();
      	if(strToken.toLowerCase().indexOf(" and ")!=-1) {
      		String temp=strToken;
      		int andPos=0;
      		out:
		    	do
		    	 	{
		    	 		andPos = temp.toLowerCase().indexOf(" and ");
			    		String strTokenAdd;
			    		if(andPos!=-1) 								
      					strTokenAdd = temp.substring(0,andPos).trim();
      				else	
      					strTokenAdd = temp.trim();
      				int delimiter2 = strTokenAdd.indexOf("=");
      				if(delimiter2!=-1) {
      						String valueAdd = strTokenAdd.substring(delimiter2+1).trim();
	 							if((valueAdd.indexOf("'")!=-1) ||(valueAdd.indexOf("\"")!=-1))
									valueAdd=valueAdd.trim().substring(1,valueAdd.trim().length()-1);
      						whereCols.add(strTokenAdd.substring(0,delimiter2).trim());
      						whereValues.add(valueAdd);
      					}
      					else {	
      						int delimiter3 = strTokenAdd.toLowerCase().indexOf(" is ");
      						whereCols.add(strTokenAdd.substring(0,delimiter3).trim());
      						whereValues.add(null);
      					}	
     					temp=temp.substring(andPos+5);
     					if(temp.toLowerCase().indexOf(" and ")==-1) {
     							strTokenAdd = temp.trim();
      					int delimiter4 = strTokenAdd.indexOf("=");
      					if(delimiter4!=-1) {
      						String valueAdd = strTokenAdd.substring(delimiter4+1).trim();
	 							if((valueAdd.indexOf("'")!=-1) ||(valueAdd.indexOf("\"")!=-1))
									valueAdd=valueAdd.trim().substring(1,valueAdd.trim().length()-1);
      						whereCols.add(strTokenAdd.substring(0,delimiter4).trim());
      						whereValues.add(valueAdd);
      					}
      					else {	
      						int delimiter3 = strTokenAdd.toLowerCase().indexOf(" is ");
      						whereCols.add(strTokenAdd.substring(0,delimiter3).trim());
      						whereValues.add(null);
      					}	
     					break out;
     					}
     							

    			}while(true);
			   
      	}
      	else {
      		int delimiter = strToken.indexOf("=");
      		if(delimiter!=-1) {
      			String value = strToken.substring(delimiter+1).trim();
	 				if((value.indexOf("'")!=-1) ||(value.indexOf("\"")!=-1))
						value=value.trim().substring(1,value.trim().length()-1);
      			whereCols.add(strToken.substring(0,delimiter).trim());
      			whereValues.add(value);
      		}
      		else {	
      			int delimiter1 = strToken.toLowerCase().indexOf(" is ");
      			whereCols.add(strToken.substring(0,delimiter1).trim());
      			whereValues.add(null);
      		}
      	}		
    	}
		columnWhereNames = new String[whereCols.size()];
    	columnWhereValues = new String[whereValues.size()];
    	whereCols.copyInto(columnWhereNames);
    	whereValues.copyInto(columnWhereValues);
    	
    	 	
    	}
	    	
	   	}	

   	}
   	else {
   		if (upperSql.lastIndexOf(" VALUES") == -1) {	
      			throw new Exception("Malformed SQL. Missing VALUES statement.");
    		}
    		sqlType=INSERT;
    		int intoPos = 0;
    		if(upperSql.indexOf(" INTO ")!=-1)
    			intoPos = upperSql.indexOf(" INTO ")+6;
    		else
    			intoPos = upperSql.indexOf("INSERT ")+7;	
    		int bracketPos = upperSql.indexOf("(");
    		int lastBracketPos = upperSql.indexOf(")");
	    	tableName = sql.substring(intoPos, bracketPos).trim();
	    	
	    	Vector cols = new Vector();
    		StringTokenizer tokenizer = new StringTokenizer(upperSql.substring(bracketPos+1,lastBracketPos), ",");
    		while (tokenizer.hasMoreTokens())
    		{
      		cols.add(tokenizer.nextToken().trim());
    		}
    		columnNames = new String[cols.size()];
    		cols.copyInto(columnNames);
    		
    		int valuesPos = upperSql.indexOf("VALUES");
	    	String endStatement = sql.substring(valuesPos+6).trim();
	    	bracketPos = endStatement.indexOf("(");
    		lastBracketPos = endStatement.lastIndexOf(")");
	    	Vector values = new Vector();
			StringTokenizer tokenizer2 = new StringTokenizer(endStatement.substring(bracketPos+1,lastBracketPos), ",");
    		while (tokenizer2.hasMoreTokens())
    		{
      		values.add(tokenizer2.nextToken().trim());
    		}
    		columnValues = new String[values.size()];
    		values.copyInto(columnValues);

   	}			
    }
    else {
    	if (upperSql.lastIndexOf(" FROM ") == -1)
    	{
      	throw new Exception("Malformed SQL. Missing FROM statement.");
    	}

    	sqlType=SELECT;
    	int fromPos = upperSql.lastIndexOf(" FROM ");
    	int wherePos=upperSql.lastIndexOf(" WHERE ");
		if(wherePos==-1)
    		tableName = sql.substring(fromPos + 6).trim();
    	else
    		tableName = sql.substring(fromPos + 6, wherePos).trim();	

    	Vector cols = new Vector();
    	StringTokenizer tokenizer = new StringTokenizer(upperSql.substring(7, fromPos), ",");

    	while (tokenizer.hasMoreTokens())
    	{
      	cols.add(tokenizer.nextToken().trim());
    	}

    	columnNames = new String[cols.size()];
    	cols.copyInto(columnNames);
    	if(wherePos!=-1)
    	 {
    	 	String strWhere=sql.substring(wherePos+7);
    	 	Vector whereCols = new Vector();
    	 	Vector whereValues = new Vector();
    		StringTokenizer tokenizerWhere = new StringTokenizer(strWhere, ",");

    	while (tokenizerWhere.hasMoreTokens())
    	{
      	String strToken = tokenizerWhere.nextToken();
      	if(strToken.toLowerCase().indexOf(" and ")!=-1) {
      		String temp=strToken;
      		int andPos=0;
      		out:
		    	do
		    	 	{
		    	 		andPos = temp.toLowerCase().indexOf(" and ");
			    		String strTokenAdd;
			    		if(andPos!=-1) 								
      					strTokenAdd = temp.substring(0,andPos).trim();
      				else	
      					strTokenAdd = temp.trim();
      				int delimiter2 = strTokenAdd.indexOf("=");
      				if(delimiter2!=-1) {
      						String valueAdd = strTokenAdd.substring(delimiter2+1).trim();
	 							if((valueAdd.indexOf("'")!=-1) ||(valueAdd.indexOf("\"")!=-1))
									valueAdd=valueAdd.trim().substring(1,valueAdd.trim().length()-1);
      						whereCols.add(strTokenAdd.substring(0,delimiter2).trim());
      						whereValues.add(valueAdd);
      					}
      					else {	
      						int delimiter3 = strTokenAdd.toLowerCase().indexOf(" is ");
      						whereCols.add(strTokenAdd.substring(0,delimiter3).trim());
      						whereValues.add(null);
      					}	
     					temp=temp.substring(andPos+5);
     					if(temp.toLowerCase().indexOf(" and ")==-1) {
     							strTokenAdd = temp.trim();
      					int delimiter4 = strTokenAdd.indexOf("=");
      					if(delimiter4!=-1) {
      						String valueAdd = strTokenAdd.substring(delimiter4+1).trim();
	 							if((valueAdd.indexOf("'")!=-1) ||(valueAdd.indexOf("\"")!=-1))
									valueAdd=valueAdd.trim().substring(1,valueAdd.trim().length()-1);
      						whereCols.add(strTokenAdd.substring(0,delimiter4).trim());
      						whereValues.add(valueAdd);
      					}
      					else {	
      						int delimiter3 = strTokenAdd.toLowerCase().indexOf(" is ");
      						whereCols.add(strTokenAdd.substring(0,delimiter3).trim());
      						whereValues.add(null);
      					}	
     					break out;
     					}
     							

    			}while(true);
			   
      	}
      	else {
      		int delimiter = strToken.indexOf("=");
      		if(delimiter!=-1) {
      			String value = strToken.substring(delimiter+1).trim();
	 				if((value.indexOf("'")!=-1) ||(value.indexOf("\"")!=-1))
						value=value.trim().substring(1,value.trim().length()-1);
      			whereCols.add(strToken.substring(0,delimiter).trim());
      			whereValues.add(value);
      		}
      		else {	
      			int delimiter1 = strToken.toLowerCase().indexOf(" is ");
      			whereCols.add(strToken.substring(0,delimiter1).trim());
      			whereValues.add(null);
      		}
      	}		
    	}
		columnWhereNames = new String[whereCols.size()];
    	columnWhereValues = new String[whereValues.size()];
    	whereCols.copyInto(columnWhereNames);
    	whereValues.copyInto(columnWhereValues);
    	
    	 	
    	}
    		
    }	
  	}
}
